#define BT_UART_DELAY 15

#include "KAmodBTM222.h"
#include <avr/io.h>
#include <util/delay.h>
#include <stdint.h>
#include <string.h>

#define F_CPU 16000000UL
#define USART_BAUD 19200ul

#define USART_UBBR_VALUE ((F_CPU/(USART_BAUD<<4))-1)

//***************************************************************************************************************
// USART functions
 
void USART_vInit(void)

{
	UBRRH = (uint8_t)(USART_UBBR_VALUE>>8);
	UBRRL = (uint8_t)USART_UBBR_VALUE;

	UCSRC = (1<<URSEL)|(1<<USBS)|(3<<UCSZ0);

	UCSRB = (1<<RXEN)|(1<<TXEN);
}


void USART_vSendByte(uint8_t u8Data)

{
	while((UCSRA&(1<<UDRE)) == 0);
	UDR = u8Data;
}


uint8_t USART_vReceiveByte()

{
	while((UCSRA&(1<<RXC)) == 0);
	return UDR;
}



// USART_vSendBuffer sends string with delays of del ms between characters
void USART_vSendBuffer(char * buffer, int del)
{
	int i = 0;
	while(buffer[i]!=0) { USART_vSendByte(buffer[i]); _delay_ms(del); i++; }
}

// USART_vReceiveBuffer gets string with delays of del ms between characters
void USART_vReceiveBuffer(char * buffer, int del)
{
	int i = 0;
	while (1)
	{
		buffer[i] = USART_vReceiveByte();
		if ((buffer[i]=='\r') | (buffer[i]=='\n')) 
		{
			buffer[i]=0;
			return;
		}
		_delay_ms(del);
		i++;
	}
}

//***************************************************************************************************************



//***************************************************************************************************************
// Functions for KAmodBTM222
//***************************************************************************************************************


// Initialization of communication interface (UART + module's reset)
int KAmodBT_Initialize()
{
	// Configure nRES 
	BT_nRES_DIR = BT_nRES_DIR | (1 << BT_nRES_PIN);
	BT_nRES_PORT |= (1 << BT_nRES_PIN);

	// Initialize USART
	USART_vInit();

	return BT_OK;	
}


// Function for enabling module's AT mode and some more
int KAmodBT_EnterATMode()
{
	char buf[32];

	// Send +++ for AT mode enabling
	USART_vSendBuffer("+++\r", 50);
	USART_vReceiveBuffer(buf, BT_UART_DELAY);
	
	
	//Disable echo, it speeds up communication, because, you don't have to wait for echo
	USART_vSendBuffer("ATE0\r", 50);
	USART_vReceiveBuffer(buf, BT_UART_DELAY);
	
	if (strcmp(buf, "OK")==0) return BT_OK; else return BT_ERR;
}

/*
void KAmodBT_RestoreDefaults()
{
	USART_vSendBuffer("ATZ0\r", BT_UART_DELAY);
}
*/

/*
void KAmodBT_ExitATMode()
{
	USART_vSendBuffer("ATE1\r", BT_UART_DELAY);
}
*/


// KAmodBT_SetName sets module's name
int KAmodBT_SetName(char * Name)
{
	char buf[32];

	sprintf(buf, "ATN=%s\r", Name);
	USART_vSendBuffer(buf, BT_UART_DELAY);
	USART_vReceiveBuffer(buf, BT_UART_DELAY);

	if (strcmp(buf, "OK")==0) return BT_OK; else return BT_ERR;
}


// KAmodBT_GetName reads module's name
int KAmodBT_GetName(char * Name)
{
	char resBuffer[16];
	USART_vSendBuffer("ATN?\r", BT_UART_DELAY);
	USART_vReceiveBuffer(Name, 0);
	USART_vReceiveBuffer(Name, 0);
	USART_vReceiveBuffer(resBuffer, 0);
	return BT_OK;
}


// KAmodBT_GetPin reads PIN (that you need to connect module to another device)
int KAmodBT_GetPin(char * Name)
{
	char resBuffer[16];
	USART_vSendBuffer("ATP?\r", BT_UART_DELAY);
	USART_vReceiveBuffer(Name, 0);
	USART_vReceiveBuffer(Name, 0);
	USART_vReceiveBuffer(resBuffer, 0);
	return BT_OK;
}


// Reset module function
void KAmodBT_Reset()
{
	BT_nRES_PORT &= !(1 << BT_nRES_PIN);
	_delay_ms(50);
	BT_nRES_PORT |= (1 << BT_nRES_PIN);
	_delay_ms(50);
}


// Sends string to module
void KAmodBT_SendBuffer(char * buffer)
{
	USART_vSendBuffer(buffer, BT_UART_DELAY);
}


